unit gui;
{************************************************************
 unit gui - this unit handles all the GUI aspects and calling
    of the name drawing functions.

 Copyright 2011 Tobias Diekershoff <tobias.diekershoff~at~gmx.net>
 for more information read README, COPYING and the DOCS

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ************************************************************}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, ExtCtrls, StdCtrls, about,
  namelist;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    GroupBox2: TGroupBox;
    FileInfos: TMemo;
    GroupBox3: TGroupBox;
    ListBox1: TListBox;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    TheName: TEdit;
    GroupBox1: TGroupBox;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    RadioGroup1: TRadioGroup;
    RadioGroup2: TRadioGroup;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ListBox1DblClick(Sender: TObject);
    procedure MenuItem4Click(Sender: TObject);
    procedure MenuItem5Click(Sender: TObject);
    procedure MenuItem6Click(Sender: TObject);
    procedure MenuItem7Click(Sender: TObject);
    procedure MenuItem8Click(Sender: TObject);
    procedure MenuItem9Click(Sender: TObject);
    procedure LoadNamefilesToList();
    procedure ReadInit();
    procedure SaveInit();
  private
    { private declarations }
    namfile : TNamFile;
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation
const
  scrybble : Array[1..104]of char = (' ',' ',' ',' ','A','A','A','A','A','A','B','B','C','C','D','D','D','D','D','E','E','E','E','E','E','E','E','E','E','E','E','E','E','E','E','E','E','F','G','G','G','G','H','H','H','I','I','I','I','I','I','J','J','K','K','L','L','L','M','M','N','N','N','N','N','N','N','N','N','N','O','O','O','O','O','O','P','P','Q','R','R','R','R','R','R','S','S','S','T','T','T','T','T','T','U','U','V','V','W','W','X','Y','Z','Z');
  alph : array[1..27] of char = (' ','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');

{ TForm1 }
procedure TForm1.ReadInit();
// load last filename from config file if that exists
var
  s : string;
begin
  if FileExists( GetAppConfigFile(False) ) then begin
    with TStringList.Create do begin
     try
       LoadFromFile( GetAppConfigFile(False) );
       s := Strings[0]
     finally
       free
     end;
    end;
    namfile := TNamFile.Create();
    namfile.filename := s;
    FileInfos.Text := namfile.comment;
    LoadNamefilesToList;
  end;
end;
procedure TForm1.SaveInit();
// write last nam file filename to the config file
begin
  if not FileExists( GetAppConfigDir(False) ) then
    CreateDir( GetAppConfigDir(False) );
  with TStringList.create() do begin
    Add(namfile.filename);
    SaveToFile( GetAppConfigFile(False) );
  end;
end;

procedure TForm1.LoadNamefilesToList();
// clear the name list and fill it with all the *.nam entries in the directory
// we just got the last file opened from.
Var Info : TSearchRec;
    dirname : string;
    i : longint;
Begin
  ListBox1.clear();
  ListBox1.Sorted:=true;
  dirname := ExtractFilePath(NamFile.Filename);
  If FindFirst (dirname+'*.nam',faAnyFile,Info)=0 then
    begin
    Repeat
      ListBox1.items.add(Info.Name);
    Until FindNext(info)<>0;
    end;
  for i := 0 to ListBox1.Items.Count-1 do
    if ListBox1.Items[i] = extractfilename(namfile.filename) then begin
      ListBox1.ItemIndex := i;
    end;
  FindClose(Info);
end;

procedure TForm1.Button1Click(Sender: TObject);
// generate a random name from the loaded name list file
var
  gender, middle : boolean;
begin
   if namfile = nil then
      showmessage('First select a *.nam file!')
   else begin
      gender := RadioButton1.Checked = True;
      middle := RadioButton4.Checked = True;
      TheName.Text := NamFile.get_random_name(gender, middle);
   end;
end;

procedure TForm1.Button2Click(Sender: TObject);
// load the selected name file from the file list
var
   filename : string;
begin
   if (ListBox1.items.count > 0) and (ListBox1.ItemIndex > -1) then begin
      filename := ExtractFilePath(NamFile.Filename)+ListBox1.Items[ListBox1.ItemIndex];
      if FileExists( filename ) then begin
         namfile := TNamFile.create();
         namfile.filename := filename;
         FileInfos.Text := namfile.comment;
         LoadNamefilesToList;
         SaveInit();
      end;
   end;
end;

procedure TForm1.Button3Click(Sender: TObject);
// open a name list file from disk, same as Ctrl+O see below
begin
  MenuItem4Click(Sender);
end;

procedure TForm1.ListBox1DblClick(Sender: TObject);
// double click on a name file list item loads that nam file as if
// loaded by clicking the "Load file" button.
begin
   Button2Click(Sender);
end;

procedure TForm1.MenuItem4Click(Sender: TObject);
// open a name list file from disk
begin
   OpenDialog1.Execute();
   namfile := TNamFile.create();
   namfile.filename := OpenDialog1.FileName;
   FileInfos.Text := namfile.comment;
   LoadNamefilesToList;
   SaveInit();
end;

procedure TForm1.MenuItem5Click(Sender: TObject);
// quit the program
begin
  close();
end;

procedure TForm1.MenuItem6Click(Sender: TObject);
// generate a random name from the loaded file, see above
begin
  Button1Click(Sender);
end;

procedure TForm1.MenuItem7Click(Sender: TObject);
// generate a random name from Scrabble letter set
var
  i,j,n, anz : longint;
  used : array[1..14]of longint;
  n_is_used : boolean;
  tmp : string;
begin
  anz := 4 + random(10);
  for i:=1 to anz do begin
    for n:=1 to 30000 do begin end;
    repeat
      n_is_used := false;
      n := random(104)+1;   // there are 104 Letters
      for j := 1 to i do    // check if n is used already
        if not n_is_used then
          n_is_used := n=used[j];
    until not n_is_used;
    used[i]:=n;
  end;
  tmp:='';
  for i:=1 to anz do tmp:=tmp+scrybble[used[i]];
  thename.text := tmp;
end;

procedure TForm1.MenuItem8Click(Sender: TObject);
begin
  form2.show()
end;

procedure TForm1.MenuItem9Click(Sender: TObject);
// generate a totally random name
VAR
  n, anz : longint;
  s : string;
begin
  anz := 4 + random(10);
  s:='';
  for n:=1 to anz do s:=s+alph[random(27)+1];
  TheName.text := s;
end;

initialization
  {$I gui.lrs}

end.

