Unit namelist;
{************************************************************
 namelist - a unit to handle GameName *.nam files with Pascal
 
 Copyright 2004 Tobias Diekershoff <tobias.diekershoff~at~gmx.net>
 for more information read README, COPYING and the DOCS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ************************************************************}

{$mode objfpc}{$H+}{$smartlink on}

Interface

Uses
  Classes, SysUtils; 
  
const
  Male          = true;    // No offense ment! just a logical swith
  Female        = false;
  WithMiddle    = true;
  WithoutMiddle = false;

type
  TNamFile = class
    private
      mycomment : string;
      surnames, malenames, femalenames, titles : TStringList;
      myfilename : string;
    public
      constructor create;
      destructor destroy; override;
      procedure set_filename(afilename:string);
      function get_filename:string;
      procedure get_names;
      function get_random_name(gender, middle : boolean) : string;
      function get_random_name : string;
      function get_comment : string;
      function get_nummales : longint;
      function get_numfemales : longint;
      function get_numlastnames : longint;
    property filename : string read get_filename write set_filename;
    property comment : string read get_comment;
    property nr_male : longint read get_nummales;
    property nr_female : longint read get_numfemales;
    property nr_last : longint read get_numlastnames;
  end;
      


Implementation

constructor TNamFile.create;
begin
  surnames := TStringList.create;
  malenames := TStringList.create;
  femalenames := TStringList.create;
  titles := TStringList.create;
  myfilename := '';
end;
destructor TNamFile.destroy;
begin
  surnames.free;
  malenames.free;
  femalenames.free;
  titles.free;
end;
procedure TNamFile.set_filename(afilename:string);
begin
  myfilename := afilename;
  get_names;
end;
function TNamFile.get_filename : string;
begin
  result := myfilename;
end;
procedure TNamFile.get_names;
var
  t : text;
  s : string;
  phase : longint;
begin
  surnames.free;
  malenames.free;
  femalenames.free;
  surnames := TStringList.create;
  malenames := TStringList.create;
  femalenames := TStringList.create;

  assign(t, myfilename);
  reset(t);
  phase := 0;
  while not eof(t) do begin
    readln(t,s);
    if (pos('COMMENT', uppercase(s))=1) then begin
      readln(t,s);
      mycomment := s;
    end;
    if (pos('MALE', uppercase(s))=1) then begin
      phase := 1;
      readln(t,s);
      readln(t,s);
    end;
    if (pos('FEMALE', uppercase(s))=1) then begin
      phase := 2;
      readln(t,s);
      readln(t,s);
    end;
    if (pos('LAST NAMES', uppercase(s))=1) then begin
      phase := 3;
      readln(t,s);
      readln(t,s);
    end;
    if (phase<>0)and(trim(s)<>'')and(s<>'END') then begin
      s := copy(trim(s), pos(' ',trim(s))+1, length(s));
      case phase of
        1 : malenames.add(s);
        2 : femalenames.add(s);
        3 : surnames.add(s);
      end;
    end;
  end;
  close(t);
end;
function TNamFile.get_random_name(gender, middle : boolean) : string;
var tmp:string;
begin
  if gender = Male then
    tmp := malenames.strings[random(malenames.count)] + ' '
  else
    tmp := femalenames.strings[random(femalenames.count)] + ' ';
  if middle = WithMiddle then begin
    if gender = Male then
      tmp := tmp + malenames.strings[random(malenames.count)] + ' '
    else
      tmp := tmp + femalenames.strings[random(femalenames.count)] + ' ';
  end;
  tmp := tmp + surnames.strings[random(surnames.count)];
  result := tmp;
end;
function TNamFile.get_random_name : string;
var gender, middle : boolean;
begin
  if ((random(10)+1)>=5) then gender := male else gender := female;
  if ((random(10)+1)>=5) then middle := withmiddle else middle := withoutmiddle;
  result := get_random_name(gender,middle);
end;
function TNamFile.get_comment : string;
begin
  result := mycomment;
end;
function TNamFile.get_nummales : longint;
begin
  result := malenames.count;
end;
function TNamFile.get_numfemales : longint;
begin
  result := femalenames.count;
end;
function TNamFile.get_numlastnames : longint;
begin
  result := surnames.count;
end;

begin
  RANDOMIZE;
End.

